@@ -293,6 +293,8 @@ terminal_window_dropdown_init (TerminalWindowDropdown *dropdown)
                               G_OBJECT (dropdown), name,
                               G_BINDING_SYNC_CREATE);
     }
+
+  gtk_widget_show_all (hbox);
 }
 
 
@@ -418,16 +420,26 @@ terminal_window_dropdown_focus_in_event (GtkWidget     *widget,
 static gboolean
 terminal_window_dropdown_can_grab (gpointer data)
 {
-  TerminalWindowDropdown *dropdown = TERMINAL_WINDOW_DROPDOWN (data);
-  GdkGrabStatus           status;
-  GdkWindow              *window;
+  GdkWindow    *window = gtk_widget_get_window (GTK_WIDGET (data));
+  GdkGrabStatus status;
 
-  window = gtk_widget_get_window (GTK_WIDGET (dropdown));
+#if GTK_CHECK_VERSION (3, 20, 0)
+  GdkSeat *seat = gdk_display_get_default_seat (gtk_widget_get_display (GTK_WIDGET (data)));
+  if (!gdk_window_is_visible (window))
+    return FALSE;
+  status = gdk_seat_grab (seat, window, GDK_SEAT_CAPABILITY_KEYBOARD, FALSE, NULL, NULL, NULL, NULL);
+#else
   status = gdk_keyboard_grab (window, FALSE, GDK_CURRENT_TIME);
+#endif
+
   if (status == GDK_GRAB_SUCCESS)
     {
       /* drop the grab */
+#if GTK_CHECK_VERSION (3, 20, 0)
+      gdk_seat_ungrab (seat);
+#else
       gdk_keyboard_ungrab (GDK_CURRENT_TIME);
+#endif
 
       return FALSE;
     }
@@ -690,6 +702,7 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
   gint               x_dest, y_dest;
   GtkRequisition     req1;
   gboolean           move_to_active;
+  gboolean           keep_above;
   gboolean           visible;
   gint               vbox_h;
   TerminalDirection  old_animation_dir = ANIMATION_DIR_NONE;
@@ -783,6 +796,11 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
     }
   else
     {
+      g_object_get (G_OBJECT (window->preferences),
+                    "dropdown-keep-above", &keep_above,
+                    NULL);
+      gtk_window_set_keep_above (GTK_WINDOW (dropdown), keep_above);
+
       /* make sure all the content fits */
       gtk_window_resize (GTK_WINDOW (dropdown), w, h);
     }
@@ -807,6 +825,7 @@ terminal_window_dropdown_toggle_real (TerminalWindowDropdown *dropdown,
       /* if the focus was lost for 0.1 second and toggle-focus is used, we had
        * focus until the shortcut was pressed, and then we hide the window */
       if (!toggle_focus
+          || dropdown->focus_out_time == 0
           || (g_get_real_time () - dropdown->focus_out_time) < G_USEC_PER_SEC / 10)
         {
           /* hide */
